package template;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import jv.geom.PgElementSet;
import jv.geom.PgPointSet;
import jv.geom.PgPolygonSet;
import jv.loader.PjImportModel;
import jv.number.PuInteger;
import jv.object.PsConfig;
import jv.object.PsDebug;
import jv.project.PgGeometryIf;
import jv.project.PjProject;
import jv.project.PvViewerIf;

public class PjTemplate extends PjProject implements ActionListener {
	private static final long serialVersionUID = 1L;
	
	/* Loader */
	static String m_defFileName = "models\\cube.obj";
	protected String m_fileName;
	protected PjImportModel m_import;

	/* Parameter */
	protected PuInteger m_intSlider;

	/**
	 * Constructor, without arguments to allow loading of project from menu.
	 */
	public PjTemplate() {
		this(PsConfig.getCodeBase() + m_defFileName);
	}

	public PjTemplate(String fileName) {
		super(fileName);
		if (fileName == null)
			fileName = "Unknown file name";
		m_fileName = fileName;
		m_import = new PjImportModel();
		m_import.setTypeOfInfoPanel(PjImportModel.SMALL_INFO_PANEL);
		m_import.setFileName(fileName);
		m_import.addActionListener(this);
		if (getClass() == PjTemplate.class) {
			init();
		}
	}

	public void method() {

	}

	/**
	 * Do initialization of data structures; method is also used to reset instances.
	 */
	public void init() {
		super.init();
		m_intSlider = new PuInteger("intSlider");
		m_intSlider.addUpdateListener(this);
		m_intSlider.setDefBounds(1, 100, 1, 1);
		m_intSlider.setDefValue(50);
		m_intSlider.init();
	}

	/**
	 * Start project, e.g. start an animation. Method is called once when project is
	 * selected in PvViewer#selectProject(). Method is optional. For example, if an
	 * applet calls the start() method of PvViewer, then PvViewer tries to invoke
	 * the start() method of the currently selected project.
	 */
	public void start() {
		super.start();
		PvViewerIf viewer = getViewer();
		m_display.setBackgroundColor(Color.WHITE);
		if (viewer != null) {
			String model = viewer.getParameter("model");
			if (model != null) {
				m_fileName = PsConfig.getCodeBase() + model;
			}
		}
		m_import.load(m_fileName);
		if (m_import.getGeometry() instanceof PgElementSet || m_import.getGeometry() instanceof PgPointSet
				|| m_import.getGeometry() instanceof PgPolygonSet) {
			addGeometry(m_import.getGeometry());
			fitDisplays();
		}
		m_geom = (PgPointSet) m_import.getGeometry();
		afterGeomLoaded();
	}

	public boolean setGeometry(PgGeometryIf geom) {
		m_geom = (PgPointSet) geom;
		return true;
	}

	/** Catch action events from the import loader when a new model was loaded. */
	public void actionPerformed(ActionEvent event) {
		if (event.getSource() == m_import) {
			if (PsDebug.NOTIFY)
				PsDebug.notify("(m_import): called");
			if (m_import.getConfirm() == PjImportModel.CONFIRM_CLICKED) {
				if (m_import.getGeometry() instanceof PgElementSet || m_import.getGeometry() instanceof PgPointSet
						|| m_import.getGeometry() instanceof PgPolygonSet) {
					addGeometry(m_import.getGeometry());
					fitDisplays();
				}
			} else if (m_import.getConfirm() == PjImportModel.CONFIRM_CANCEL) {
				fitDisplays();
			} else if (m_import.getConfirm() == PjImportModel.CONFIRM_OK) {
				m_geom = (PgPointSet) m_import.getGeometry();
				if (!addGeometry(m_geom))
					return;
			}
			fitDisplays();
		}
		m_geom = (PgPointSet) m_import.getGeometry();
		afterGeomLoaded();
	}

	/**
	 * Called in start(), actionPerformed() and load().
	 */
	public void afterGeomLoaded() {
		System.out.println("Loaded geometry: " + m_fileName);
		m_geom = (PgPointSet) m_import.getGeometry();
	}

	/**
	 * 
	 * @param filepath
	 */
	public void load(String filepath) {
		m_fileName = filepath;
		System.out.println(m_fileName);
		m_import.load(m_fileName);
		if (m_import.getGeometry() instanceof PgElementSet || m_import.getGeometry() instanceof PgPointSet
				|| m_import.getGeometry() instanceof PgPolygonSet) {
			addGeometry(m_import.getGeometry());
			fitDisplays();
		}
		afterGeomLoaded();
	}

	/**
	 * Update method of project to react on changes in its panel or of its children.
	 * This method is optional, but required if project is parent of a child.
	 * Project becomes parent if child calls <code>child.setParent(this)</code> with
	 * this project as argument. For example, see the constructor of MyProject.
	 * Project must react on child events, or forward them to its superclass.
	 * <p>
	 * Catch events of integer children and recompute surface.
	 */
	public boolean update(Object event) {
		if (event == m_intSlider) {
			System.out.println(" Number of iterations : " + m_intSlider.getValue());
			return true;
		}

		return super.update(event);
	}
}
